package org.fuseri.modulecertificate.service;


import org.fuseri.model.dto.certificate.CertificateCreateDto;
import org.fuseri.model.dto.certificate.CertificateSimpleDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class CertificateFacade {

    private final CertificateService certificateService;
    private final CertificateMapper certificateMapper;

    @Autowired
    public CertificateFacade(CertificateService certificateService, CertificateMapper certificateMapper) {
        this.certificateService = certificateService;
        this.certificateMapper = certificateMapper;
    }

    @Cacheable(cacheNames = "certificates", key = "#id")
    @Transactional(readOnly = true)
    public CertificateSimpleDto findById(Long id) {
        return certificateMapper.mapToSimpleDto(certificateService.findById(id));
    }

    @Transactional()
    public CertificateSimpleDto generate(CertificateCreateDto certificateCreateDto) {
        return certificateMapper.mapToSimpleDto(certificateService.save(certificateMapper.mapToCertificate(certificateCreateDto)));
    }

    @Cacheable(cacheNames = "certificates", key = "#userId")
    @Transactional(readOnly = true)
    public List<CertificateSimpleDto> findByUserId(Long userId) {
        return certificateMapper.mapToList(certificateService.findByUserId(userId));
    }

    @Cacheable(cacheNames = "certificates", key = "#userId")
    @Transactional(readOnly = true)
    public List<CertificateSimpleDto> findByUserIdAndCourseId(Long userId, Long courseId) {
        return certificateMapper.mapToList(certificateService.findByUserIdAndCourseId(userId, courseId));
    }

    @Transactional(readOnly = true)
    public Page<CertificateSimpleDto> findAll(Pageable pageable) {
        return certificateMapper.mapToPageDto(certificateService.findAll(pageable));
    }

    @Transactional(readOnly = true)
    public void deleteCertificate(Long certificateId) {
        certificateService.delete(certificateId);
    }
}
